from ._novel import \
    _package, _process, _update, _novel, _info, _clean_novel, _delete_novel, _url, _add_url, _remove_url
# from ._manage import _manage
from ._config import _set_novel_dir
from ._version import _version
